@@ -1,21 +1,23 @@
 use anchor_client::Cluster;
 use anchor_syn::idl::Idl;
-use anyhow::{anyhow, Error, Result};
-use clap::{ArgEnum, Clap};
+use anyhow::{anyhow, Context, Error, Result};
+use clap::{ArgEnum, Parser};
 use heck::SnakeCase;
 use serde::{Deserialize, Serialize};
+use solana_cli_config::{Config as SolanaConfig, CONFIG_FILE};
 use solana_sdk::pubkey::Pubkey;
 use solana_sdk::signature::{Keypair, Signer};
 use std::collections::BTreeMap;
 use std::convert::TryFrom;
 use std::fs::{self, File};
+use std::io;
 use std::io::prelude::*;
 use std::ops::Deref;
 use std::path::Path;
 use std::path::PathBuf;
 use std::str::FromStr;
 
-#[derive(Default, Debug, Clap)]
+#[derive(Default, Debug, Parser)]
 pub struct ConfigOverride {
     /// Cluster override.
     #[clap(global = true, long = "provider.cluster")]
@@ -55,9 +57,10 @@ pub struct Manifest(cargo_toml::Manifest);
 
 impl Manifest {
     pub fn from_path(p: impl AsRef<Path>) -> Result<Self> {
-        cargo_toml::Manifest::from_path(p)
+        cargo_toml::Manifest::from_path(&p)
             .map(Manifest)
-            .map_err(Into::into)
+            .map_err(anyhow::Error::from)
+            .with_context(|| format!("Error reading manifest from path: {}", p.as_ref().display()))
     }
 
     pub fn lib_name(&self) -> Result<String> {
@@ -99,8 +102,14 @@ impl Manifest {
         let mut cwd_opt = Some(start_from.as_path());
 
         while let Some(cwd) = cwd_opt {
-            for f in fs::read_dir(cwd)? {
-                let p = f?.path();
+            for f in fs::read_dir(cwd).with_context(|| {
+                format!("Error reading the directory with path: {}", cwd.display())
+            })? {
+                let p = f
+                    .with_context(|| {
+                        format!("Error reading the directory with path: {}", cwd.display())
+                    })?
+                    .path();
                 if let Some(filename) = p.file_name() {
                     if filename.to_str() == Some("Cargo.toml") {
                         let m = WithPath::new(Manifest::from_path(&p)?, p);
@@ -162,7 +171,12 @@ impl WithPath<Config> {
             let cargo = Manifest::from_path(&path.join("Cargo.toml"))?;
             let lib_name = cargo.lib_name()?;
             let version = cargo.version();
-            let idl = anchor_syn::idl::file::parse(path.join("src/lib.rs"), version)?;
+            let idl = anchor_syn::idl::file::parse(
+                path.join("src/lib.rs"),
+                version,
+                self.features.seeds,
+                false,
+            )?;
             r.push(Program {
                 lib_name,
                 path,
@@ -243,6 +257,7 @@ impl<T> std::ops::DerefMut for WithPath<T> {
 pub struct Config {
     pub anchor_version: Option<String>,
     pub solana_version: Option<String>,
+    pub features: FeaturesConfig,
     pub registry: RegistryConfig,
     pub provider: ProviderConfig,
     pub programs: ProgramsConfig,
@@ -251,6 +266,12 @@ pub struct Config {
     pub test: Option<Test>,
 }
 
+#[derive(Default, Clone, Debug, Serialize, Deserialize)]
+pub struct FeaturesConfig {
+    #[serde(default)]
+    pub seeds: bool,
+}
+
 #[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct RegistryConfig {
     pub url: String,
@@ -284,7 +305,7 @@ pub struct WorkspaceConfig {
     pub types: String,
 }
 
-#[derive(ArgEnum, Clap, Clone, PartialEq, Debug)]
+#[derive(ArgEnum, Parser, Clone, PartialEq, Debug)]
 pub enum BootstrapMode {
     None,
     Debian,
@@ -327,8 +348,14 @@ impl Config {
         let mut cwd_opt = Some(_cwd.as_path());
 
         while let Some(cwd) = cwd_opt {
-            for f in fs::read_dir(cwd)? {
-                let p = f?.path();
+            for f in fs::read_dir(cwd).with_context(|| {
+                format!("Error reading the directory with path: {}", cwd.display())
+            })? {
+                let p = f
+                    .with_context(|| {
+                        format!("Error reading the directory with path: {}", cwd.display())
+                    })?
+                    .path();
                 if let Some(filename) = p.file_name() {
                     if filename.to_str() == Some("Anchor.toml") {
                         let cfg = Config::from_path(&p)?;
@@ -344,12 +371,9 @@ impl Config {
     }
 
     fn from_path(p: impl AsRef<Path>) -> Result<Self> {
-        let mut cfg_file = File::open(&p)?;
-        let mut cfg_contents = String::new();
-        cfg_file.read_to_string(&mut cfg_contents)?;
-        let cfg = cfg_contents.parse()?;
-
-        Ok(cfg)
+        fs::read_to_string(&p)
+            .with_context(|| format!("Error reading the file with path: {}", p.as_ref().display()))?
+            .parse()
     }
 
     pub fn wallet_kp(&self) -> Result<Keypair> {
@@ -362,6 +386,7 @@ impl Config {
 struct _Config {
     anchor_version: Option<String>,
     solana_version: Option<String>,
+    features: Option<FeaturesConfig>,
     programs: Option<BTreeMap<String, BTreeMap<String, serde_json::Value>>>,
     registry: Option<RegistryConfig>,
     provider: Provider,
@@ -389,6 +414,7 @@ impl ToString for Config {
         let cfg = _Config {
             anchor_version: self.anchor_version.clone(),
             solana_version: self.solana_version.clone(),
+            features: Some(self.features.clone()),
             registry: Some(self.registry.clone()),
             provider: Provider {
                 cluster: format!("{}", self.provider.cluster),
@@ -417,12 +443,13 @@ impl FromStr for Config {
         Ok(Config {
             anchor_version: cfg.anchor_version,
             solana_version: cfg.solana_version,
+            features: cfg.features.unwrap_or_default(),
             registry: cfg.registry.unwrap_or_default(),
             provider: ProviderConfig {
                 cluster: cfg.provider.cluster.parse()?,
                 wallet: shellexpand::tilde(&cfg.provider.wallet).parse()?,
             },
-            scripts: cfg.scripts.unwrap_or_else(BTreeMap::new),
+            scripts: cfg.scripts.unwrap_or_default(),
             test: cfg.test,
             programs: cfg.programs.map_or(Ok(BTreeMap::new()), deser_programs)?,
             workspace: cfg.workspace.unwrap_or_default(),
@@ -430,6 +457,16 @@ impl FromStr for Config {
     }
 }
 
+pub fn get_solana_cfg_url() -> Result<String, io::Error> {
+    let config_file = CONFIG_FILE.as_ref().ok_or_else(|| {
+        io::Error::new(
+            io::ErrorKind::NotFound,
+            "Default Solana config was not found",
+        )
+    })?;
+    SolanaConfig::load(config_file).map(|config| config.json_rpc_url)
+}
+
 fn ser_programs(
     programs: &BTreeMap<Cluster, BTreeMap<String, ProgramDeployment>>,
 ) -> BTreeMap<String, BTreeMap<String, serde_json::Value>> {
@@ -493,7 +530,6 @@ fn deser_programs(
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct Test {
     pub genesis: Option<Vec<GenesisEntry>>,
-    pub clone: Option<Vec<CloneEntry>>,
     pub validator: Option<Validator>,
     pub startup_wait: Option<i32>,
 }
@@ -512,11 +548,25 @@ pub struct CloneEntry {
     pub address: String,
 }
 
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct AccountEntry {
+    // Base58 pubkey string.
+    pub address: String,
+    // Name of JSON file containing the account data.
+    pub filename: String,
+}
+
 #[derive(Debug, Default, Clone, Serialize, Deserialize)]
 pub struct Validator {
+    // Load an account from the provided JSON file
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub account: Option<Vec<AccountEntry>>,
     // IP address to bind the validator ports. [default: 0.0.0.0]
     #[serde(default = "default_bind_address")]
     pub bind_address: String,
+    // Copy an account from the cluster referenced by the url argument.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub clone: Option<Vec<CloneEntry>>,
     // Range to use for dynamically assigned ports. [default: 1024-65535]
     #[serde(skip_serializing_if = "Option::is_none")]
     pub dynamic_port_range: Option<String>,
@@ -585,15 +635,22 @@ impl Program {
 
     // Lazily initializes the keypair file with a new key if it doesn't exist.
     pub fn keypair_file(&self) -> Result<WithPath<File>> {
-        fs::create_dir_all("target/deploy/")?;
+        let deploy_dir_path = "target/deploy/";
+        fs::create_dir_all(deploy_dir_path)
+            .with_context(|| format!("Error creating directory with path: {}", deploy_dir_path))?;
         let path = std::env::current_dir()
             .expect("Must have current dir")
             .join(format!("target/deploy/{}-keypair.json", self.lib_name));
         if path.exists() {
-            return Ok(WithPath::new(File::open(&path)?, path));
+            return Ok(WithPath::new(
+                File::open(&path)
+                    .with_context(|| format!("Error opening file with path: {}", path.display()))?,
+                path,
+            ));
         }
         let program_kp = Keypair::generate(&mut rand::rngs::OsRng);
-        let mut file = File::create(&path)?;
+        let mut file = File::create(&path)
+            .with_context(|| format!("Error creating file with path: {}", path.display()))?;
         file.write_all(format!("{:?}", &program_kp.to_bytes()).as_bytes())?;
         Ok(WithPath::new(file, path))
     }
